{% extends 'generic/object_list.html' %}
{% load helpers %}

{% block buttons %}
    <a href="{% url 'plugins:nautobot_version_control:pull_request_add' %}" class="btn btn-primary">
        <span class="mdi mdi-plus-thick" aria-hidden="true"></span> Add
    </a>
{% endblock %}

{% block title %}
Pull Requests
{% endblock %}

{{ form.as_p }}
