{% load django_tables2 %}

<div class="conflicts" >
    {% if conflicts %}

        {% if conflicts.conflicts and conflicts.conflicts.rows %}
            <div class="row">
                <div class="col-md-12">
                    <h3 id="conflicts-heading">Merge Conflicts</h3>
                    {% include 'nautobot_version_control/conflicts_panel.html' with table=conflicts.conflicts %}
                </div>
            </div>
        {% endif %}

        {% if conflicts.violations and conflicts.violations.rows %}
            <div class="row">
                <div class="col-md-12">
                    <h3 id="violations-heading">Merge Constraint Violations</h3>
                    {% include 'nautobot_version_control/conflicts_panel.html' with table=conflicts.violations %}
                </div>
            </div>
        {% endif %}

    {% endif %}
</div>
