# Generated by Django 3.1.13 on 2021-08-18 17:08

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("nautobot_version_control", "0006_auto_20210817_0251"),
    ]

    operations = [
        migrations.AlterField(
            model_name="pullrequest",
            name="created_at",
            field=models.DateTimeField(auto_now_add=True, null=True),
        ),
    ]
