{% extends 'generic/object_bulk_delete.html' %}
{% load helpers %}
{% load render_table from django_tables2 %}

{% block title %}Delete {{ table.rows|length }} {{ obj_type_plural|bettertitle }}?{% endblock %}

{% block content %}
    <div class="row">
        <div class="col-md-8 col-md-offset-2">
            <div class="panel panel-danger">
                <div class="panel-heading"><strong>Confirm Bulk Deletion</strong></div>
                <div class="panel-body">
                    <p>
                        <strong>Warning:</strong>
                        The following operation will delete {{ table.rows|length }} pull request(s).
                        Please carefully review the {{ obj_type_plural }} to be deleted and confirm below.
                    </p>
                    {% block message_extra %}{% endblock %}
                </div>
            </div>
        </div>
    </div>
    <div class="row">
        <div class="col-md-8 col-md-offset-2">
            <div class="panel panel-default">
                {% render_table table 'inc/table.html' %}
            </div>
        </div>
    </div>
    <div class="row">
        <div class="col-md-6 col-md-offset-3">
            <form action="" method="post" class="form">
                {% csrf_token %}
                {% for field in form.hidden_fields %}
                    {{ field }}
                {% endfor %}
                <div class="text-center">
                    <button type="submit" name="_confirm" class="btn btn-danger">Delete these {{ table.rows|length }} {{ obj_type_plural }}</button>
                    <a href="{{ return_url }}" class="btn btn-default">Cancel</a>
                </div>
            </form>
        </div>
    </div>
{% endblock %}