{% extends 'generic/object_edit.html' %}
{% load form_helpers %}

{% block form %}
<div class="panel panel-default">
    <div class="panel-heading"><strong>Pull Request Review</strong></div>
    <div class="panel-body">
        {% render_field form.pull_request %}
        {% render_field form.summary %}
        {% render_field form.state %}
    </div>
</div>
{% endblock %}
