"""Constants.py defines several important constants used throughout the plugin."""

# TODO: move these to settings?


DB_NAME = "nautobot"

GLOBAL_DB = "global"

DOLT_DEFAULT_BRANCH = "main"

DOLT_BRANCH_KEYWORD = "dolt-branch"
