# Generated by Django 3.1.12 on 2021-06-24 19:37

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("nautobot_version_control", "0001_initial"),
    ]

    operations = [
        migrations.CreateModel(
            name="BranchMeta",
            fields=[
                ("branch", models.TextField(primary_key=True, serialize=False)),
                ("source_branch", models.TextField()),
                ("created", models.DateTimeField(auto_now_add=True, null=True)),
                (
                    "author",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                "db_table": "nautobot_version_control_branchmeta",
            },
        ),
    ]
