{% extends 'nautobot_version_control/pull_request/base.html' %}
{% load buttons %}
{% load custom_links %}
{% load helpers %}
{% load plugins %}
{% load static %}
{% load render_table from django_tables2 %}

{% block content %}
<form method="post">
    {% csrf_token %}
    <div class="panel panel-default">
        <div class="panel-heading">
            <strong>Commits in {{ object.title }}</strong>
        </div>
        {% render_table commits_table 'inc/table.html' %}
        <div class="panel-footer noprint">
            <button type="submit" name="_revert" formaction="{% url 'plugins:nautobot_version_control:commit_revert' %}" class="btn btn-warning btn-sm">
                <span class="mdi mdi-undo-variant" aria-hidden="true"></span> Revert Selected Commits
            </button>
            <div class="clearfix"></div>
        </div>
    </div>
</form>
{% endblock %}
